--- picture.el	Sun Dec  7 07:32:26 1997
+++ picture.el.orig	Thu Jun  3 20:23:26 1999
@@ -212,6 +212,41 @@
     ;; XEmacs addition:
     (setq zmacs-region-stays nil)))
 
+(defun picture-insert-after-hook (begin end)
+  "Called as a hook after a string is inserted into the current buffer.
+Two arguments BEGIN and END point the head and tail of just inserted string."
+  (let ((str (buffer-substring begin end)))
+    (delete-region begin end)
+    (picture-insert-string str)))
+
+(defun picture-insert-string (str)
+  "Insert characters in STR one by one in place of character previously
+at the cursor while moving cursor in the direction you previously specified
+with the commands picture-movement-right, picture-movement-up, etc.
+Do \\[command-apropos]  picture-movement  to see those commands."
+  (interactive "sInsert string: ")
+  (let  ((len (length str)))
+    (if (> len 0)
+	(let* ((idx 0)
+	       (c (sref str idx))
+	       (width (char-width c))
+	       clm)
+	  (while idx				; Set to nil at the end of loop
+	    (setq clm (current-column))
+	    (delete-text-in-column clm (+ clm width))
+	    (insert c)
+	    (forward-char -1)
+	    (setq idx (+ idx (char-bytes c)))
+	    (if (< idx len)
+		(setq c (sref str idx) width (char-width c))
+	      (setq idx nil))
+	    (if (/= picture-vertical-step 0)
+		(picture-move)
+	      (if (> picture-horizontal-step 0)
+		  (forward-char 1)
+		(move-to-column-force (- clm width))))
+	    )))))
+
 (defun picture-clear-column (arg)
   "Clear out ARG columns after point without moving."
   (interactive "p")
@@ -538,6 +573,7 @@
 (defvar picture-mode-old-mode-name)
 (defvar picture-mode-old-major-mode)
 (defvar picture-mode-old-truncate-lines)
+(defvar picture-mode-old-self-insert-after-hook)
 
 ;;;###autoload
 (defun picture-mode ()
@@ -619,6 +655,10 @@
     (setq picture-mode-old-truncate-lines truncate-lines)
     (setq truncate-lines t)
 
+    (make-local-variable 'picture-mode-old-self-insert-after-hook)
+    (setq picture-mode-old-self-insert-after-hook self-insert-after-hook)
+    (setq self-insert-after-hook 'picture-insert-after-hook)
+
     ;; XEmacs addition:
     (make-local-variable 'mouse-track-rectangle-p)
     (setq mouse-track-rectangle-p t)
@@ -648,6 +688,9 @@
     (setq major-mode picture-mode-old-major-mode)
     (kill-local-variable 'tab-stop-list)
     (setq truncate-lines picture-mode-old-truncate-lines)
+
+    (setq self-insert-after-hook picture-mode-old-self-insert-after-hook)
+
     ;; XEmacs change/addition:
     (kill-local-variable 'mouse-track-rectangle-p)
     (redraw-modeline)))
